@@ -1,14 +1,13 @@
 import dataclasses
-import inspect
-from typing import Any, Callable, List, Optional, Type, TypeVar, Union, overload
+from typing import Callable, List, Optional, Type, Union
 
-from .arguments import get_arguments_from_resolver
 from .permission import BasePermission
+from .types.fields.resolver import StrawberryResolver
 from .types.types import FederationFieldParams, FieldDefinition
 from .utils.str_converters import to_camel_case
 
 
-F = TypeVar("F", bound=Callable[..., Any])
+_RESOLVER_TYPE = Union[StrawberryResolver, Callable]
 
 
 class StrawberryField(dataclasses.Field):
@@ -27,29 +26,24 @@ class StrawberryField(dataclasses.Field):
             metadata=None,
         )
 
-    def __call__(self, resolver: F) -> F:
-        """Migrate the field definition to the resolver"""
+    def __call__(self, resolver: _RESOLVER_TYPE) -> "StrawberryField":
+        """Add a resolver to the field"""
 
-        field_definition = self._field_definition
-        # note that field_definition.name is finalized in type_resolver._get_fields
+        # Allow for StrawberryResolvers or bare functions to be provided
+        if not isinstance(resolver, StrawberryResolver):
+            resolver = StrawberryResolver(resolver)
 
-        field_definition.origin_name = resolver.__name__
-        field_definition.origin = resolver
-        field_definition.base_resolver = resolver
-        field_definition.arguments = get_arguments_from_resolver(resolver)
-        field_definition.type = resolver.__annotations__.get("return", None)
+        self._field_definition.origin_name = resolver.name
+        self._field_definition.origin = resolver.wrapped_func
+        self._field_definition.base_resolver = resolver
+        self._field_definition.arguments = resolver.arguments
+        self._field_definition.type = resolver.type
 
-        if not inspect.ismethod(resolver):
-            # resolver is a normal function
-            resolver._field_definition = field_definition  # type: ignore
-        else:
-            # resolver is a bound method and immutable (most likely a
-            # classmethod or an instance method). We need to monkeypatch its
-            # underlying .__func__ function
-            # https://stackoverflow.com/a/7891681/8134178
-            resolver.__func__._field_definition = field_definition  # type:ignore
+        # Don't add field to __init__ or __repr__
+        self.init = False
+        self.repr = False
 
-        return resolver
+        return self
 
     def __setattr__(self, name, value):
         if name == "type":
@@ -65,23 +59,8 @@ class StrawberryField(dataclasses.Field):
         return super().__setattr__(name, value)
 
 
-@overload
 def field(
-    resolver: Callable,
-    *,
-    name: Optional[str] = None,
-    is_subscription: bool = False,
-    description: Optional[str] = None,
-    permission_classes: Optional[List[Type[BasePermission]]] = None,
-    federation: Optional[FederationFieldParams] = None,
-    deprecation_reason: Optional[str] = None,
-) -> Any:
-    ...
-
-
-@overload
-def field(
-    resolver: Optional[None] = None,
+    resolver: Optional[_RESOLVER_TYPE] = None,
     *,
     name: Optional[str] = None,
     is_subscription: bool = False,
@@ -90,19 +69,6 @@ def field(
     federation: Optional[FederationFieldParams] = None,
     deprecation_reason: Optional[str] = None,
 ) -> StrawberryField:
-    ...
-
-
-def field(
-    resolver: Optional[Callable] = None,
-    *,
-    name: Optional[str] = None,
-    is_subscription: bool = False,
-    description: Optional[str] = None,
-    permission_classes: Optional[List[Type[BasePermission]]] = None,
-    federation: Optional[FederationFieldParams] = None,
-    deprecation_reason: Optional[str] = None,
-) -> Union[Any, StrawberryField]:
     """Annotates a method or property as a GraphQL field.
 
     This is normally used inside a type declaration:
@@ -120,11 +86,9 @@ def field(
 
     field_definition = FieldDefinition(
         origin_name=None,  # modified by resolver in __call__
-        name=name,  # modified by resolver in __call__
-        type=None,  # type: ignore
-        origin=resolver,  # type: ignore
+        name=name,
+        type=None,
         description=description,
-        base_resolver=resolver,
         is_subscription=is_subscription,
         permission_classes=permission_classes or [],
         arguments=[],  # modified by resolver in __call__
